/*

 Kevin Moore
 Brigham Young University
 January 24th 2015
 
 Propeller wind tunnel testing code: This code runs a PID loop on the rpm and throttle.
 A desired rpm value is input into the serial command bar, startup value is 0 rpm.
*/

#include <Servo.h>
#include <PID_v1.h>

int interval = 1000;   // 1/4 second for recording motor pulses
Servo myservo;          // servo on digital pin 9
double sensorvalue=0;        // sensor reading, used for running average
unsigned long lastRpmTime = 0;

//Tach Interrupt
int TachIn = 2;                                //Pin serving as external interrupt (pin 2) to count the pulses 
volatile unsigned long siPulseCounter=0;                       //Variable keeping track of pulse counts
unsigned long rpm = 0;
int val = 0;

unsigned long rpmHist[5];
unsigned long rpmOut[20];

//PID
//Define Variables we'll be connecting to
double Setpoint, Input, Output;

//Define the aggressive and conservative Tuning Parameters
double aggKp=4, aggKi=0.2, aggKd=1;
double consKp=0.0012, consKi=0.001, consKd=0;

//Specify the links and initial tuning parameters
PID myPID(&Input, &Output, &Setpoint, consKp, consKi, consKd, DIRECT);
// Setup Tach Interupt

void count()                                    // Function called by AttachInterrupt at interrupt at int 0
{

  siPulseCounter++;                        //increment the pulse count

}

void setup() {
  // Notify by tone that system is armed, the setup routine runs once when you press reset:
  // initialize serial communication at 9600 bits per second:
  Serial.begin(9600);
  //Serial.setTimeout(50);
  myservo.attach(9);  // attaches the servo on pin 9 to the servo object
  myservo.writeMicroseconds(1200);



  //Tach Interrupt Stuff
  pinMode(TachIn, INPUT);                  // Set Pulsepin to accept inputs
  digitalWrite(TachIn, LOW);  
  attachInterrupt(0, count, RISING);          // Caputres external interrupt 0  at iPulsepin at rising edge and calls funtion count defined below 

  //PID Stuff
  //initialize the variables we're linked to
  Input = rpm;
  Setpoint = 0;

  //turn the PID on
  myPID.SetMode(AUTOMATIC);

  for(int i=0; i<5; i++){
    rpmHist[i] = 0; 
  }
  
  for(int i=0; i<20; i++){
    rpmOut[i] = 0; 
  }
}
void loop(){
  unsigned long currentTime = millis();

  if(currentTime - lastRpmTime > 100)
  {
    //Calculate RPM
    rpm = siPulseCounter*464;
//    if (rpm > 500){
//      rpm = rpm - 500;
//    }
    siPulseCounter = 0;
    lastRpmTime = currentTime;

    //Average RPM values
    for(int i=4; i>1; i--){
      rpmHist[i] = rpmHist[i-1]; 
    }
    rpmHist[0] = rpm;

    long rpmSum = 0;

  rpmSum = rpmHist[0] + rpmHist[1] + rpmHist[2] + rpmHist[3] + rpmHist[4];

    rpmSum = rpmSum/5;

    //PID CONTROLLS RPM
    Input = rpmSum;

    //    double gap = abs(Setpoint-Input); //distance away from setpoint
    //    if(gap<10)
    //    {  //we're close to setpoint, use conservative tuning parameters
    //      myPID.SetTunings(consKp, consKi, consKd);
    //    }
    //    else
    //    {
    //      //we're far from setpoint, use aggressive tuning parameters
    //      myPID.SetTunings(aggKp, aggKi, aggKd);
    //    }

    myPID.Compute();
    unsigned int servoOutput = map(Output, 0, 255, 1200, 1900);
    
if (servoOutput >1600){
servoOutput=1600;}

//Average Output values
    for(int i=19; i>1; i--){
      rpmOut[i] = rpmOut[i-1]; 
    }
    rpmOut[0] = Input;

    long rpmSumOut = 0;

  rpmSumOut = rpmOut[0] + rpmOut[1] + rpmOut[2] + rpmOut[3] + rpmOut[4]+rpmOut[5] + rpmOut[6] + rpmOut[7] + rpmOut[8] + rpmOut[9]+rpmOut[10] + rpmOut[11] + rpmOut[12] + rpmOut[13] + rpmOut[14]+rpmOut[15] + rpmOut[16] + rpmOut[17] + rpmOut[18] + rpmOut[19];

    rpmSumOut = rpmSumOut/20;


    myservo.writeMicroseconds(servoOutput); 

    Serial.print("RPM: "); 
    Serial.print(Input);
    Serial.print("RPM: "); 
    Serial.print(rpmSumOut);
    Serial.print("| Servo O: "); 
    Serial.println(servoOutput);

  }

  if (Serial.available() > 0)
  {
    Setpoint = Serial.parseInt();        // new command in 1.0 forward

    char junk;

    while (Serial.available() > 0)  // .parseFloat() can leave non-numeric characters
    { 
      junk = Serial.read() ; 
    }      // clear the keyboard buffer
  }

  
}





